"=====================================================================
" cream-bugreport.vim -- Debug info report
"
" Cream -- An easy-to-use configuration of the famous Vim  text editor
" [ http://cream.sourceforge.net ] Copyright (C) 2002-2004  Steve Hall
" 
" License:
" This program is free software; you can redistribute it and/or modify
" it under the terms of the GNU General Public License as published by
" the Free Software Foundation; either version 2 of  the  License,  or
" (at your option) any later version.
" [ http://www.gnu.org/licenses/gpl.html ]
" 
" This program is distributed in the hope that it will be useful,  but
" WITHOUT  ANY  WARRANTY;  without  even  the  implied   warranty   of
" MERCHANTABILITY or FITNESS FOR A PARTICULAR  PURPOSE.  See  the  GNU
" General Public License for more details.
" 
" You should have received a copy of the GNU  General  Public  License
" along with  this  program;  if  not,  write  to  the  Free  Software
" Foundation,  Inc.,  59  Temple  Place  -  Suite  330,   Boston,   MA
" 02111-1307, USA.
"

" Updated: 2003-04-01T14:58:41-0500
"

" register as a Cream add-on
if exists("$CREAM")
	call Cream_addon_register(
	\ 'Cream Config Info', 
	\ "Create report of Cream and Vim application information", 
	\ "Create report of Cream and Vim application information", 
	\ 'Cream Config Info', 
	\ 'call Cream_bugreport()', 
	\ '<Nil>'
	\ )
endif

function! Cream_bugreport()

	" utility functions
	function! <SID>CheckDir(n)
		if isdirectory(a:n)
			echo 'directory "' . a:n . '" exists'
		else
			echo 'directory "' . a:n . '" does NOT exist'
		endif
	endfunction
	function! <SID>CheckFile(n)
		if filereadable(a:n)
			echo '"' . a:n . '" is readable'
		else
			echo '"' . a:n . '" is NOT readable'
		endif
	endfunction

	let n = confirm(
		\ "This process may take a few seconds. Press Ok to proceed...\n" .
		\ "\n", "&Ok\n&Cancel", 1, "Info")
	if n != 1
		return
	endif
	
	silent call Cream_bugreport_info()
	call Cream_file_new()
	put! x

	delfunction <SID>CheckDir
	delfunction <SID>CheckFile

	setlocal foldmethod=marker

endfunction

function! Cream_bugreport_info()
" dumps all info to @x register

	let @x = ""
	redir @x

	let mymore = &more
	set nomore

	echo "Report generated: " . strftime("%Y-%m-%dT%H:%M:%S")

	" system info
	echo "\nSystem Info ----------------------------------------------------- {{{1\n"
	if     has("unix")
		echo system("uname -a")
	endif

    let myshellslash = &shellslash
    set noshellslash
	silent! echo system("set")
    let &shellslash = myshellslash

	" vim version info
	echo "\nVersion --------------------------------------------------------- {{{1\n"
	version

	" paths, files and directories
	echo "\nDirectories and Files ------------------------------------------- {{{1\n"
	echo '$VIM = "' . $VIM . '"'
	call <SID>CheckDir($VIM)
	echo '$VIMRUNTIME = "' . $VIMRUNTIME . '"'
	call <SID>CheckDir($VIMRUNTIME)
	call <SID>CheckFile(&helpfile)
	call <SID>CheckFile(fnamemodify(&helpfile, ":h") . "/tags")
	call <SID>CheckFile($VIMRUNTIME . "/menu.vim")
	call <SID>CheckFile($VIMRUNTIME . "/filetype.vim")
	call <SID>CheckFile($VIMRUNTIME . "/syntax/synload.vim")

	" settings
	echo "\nSettings -------------------------------------------------------- {{{1\n"
	set all
	set termcap

	" autocommands
	if has("autocmd")
		echo "\nAutocommands ------------------------------------------------ {{{1\n"
		autocmd
	endif

	" mappings
	echo "\nNormal mode mappings -------------------------------------------- {{{1\n"
	nmap

	echo "\nVisual mode mappings -------------------------------------------- {{{1\n"
	vmap

	echo "\nInsert mode mappings -------------------------------------------- {{{1\n"
	imap

	echo "\nCommand-line mode mappings -------------------------------------- {{{1\n"
	cmap

	echo "\nAbbreviations --------------------------------------------------- {{{1\n"
	abbreviate

	echo "\nHighlighting ---------------------------------------------------- {{{1\n"
	highlight

	echo "\nVariables ------------------------------------------------------- {{{1\n"
	let

	" Cream info
	echo "\nCream ----------------------------------------------------------- {{{1\n"
	if exists("g:cream_version") | echo "g:cream_version = " . g:cream_version | endif
	if exists("g:cream_version_str") | echo "g:cream_version_str = " . g:cream_version_str | endif
	if exists("g:cream_updated") | echo "g:cream_updated = " . g:cream_updated | endif
	if exists("g:cream_dev") | echo "g:cream_dev = " . g:cream_dev | endif
	if exists("g:cream_mail") | echo "g:cream_mail = " . g:cream_mail | endif

	echo "\n$VIM/*"
	echo globpath($VIM, "*")

	echo "\n$VIMRUNTIME/*"
	echo globpath($VIMRUNTIME, "*")

	echo "\n$CREAM/*"
	echo globpath($CREAM, "*")
	echo "\n$CREAM/addons/*"
	echo globpath($CREAM, "addons/*")
	echo "\n$CREAM/bitmaps/*"
	echo globpath($CREAM, "bitmaps/*")



	echo "\n  1}}}\n"
	echo "\n  vim:foldmethod=marker\n"
	redir END

	let &more = mymore

endfunction

