"=====================================================================
" cream-cream-release.vim -- developer release packager
"
" Cream -- An easy-to-use configuration of the famous Vim text  editor
" [ http://cream.sourceforge.net ] Copyright (C) 2002-2004  Steve Hall
"
" License:
" This program is free software; you can redistribute it and/or modify
" it under the terms of the GNU General Public License as published by
" the Free Software Foundation; either version 2 of  the  License,  or
" (at your option) any later version.
" [ http://www.gnu.org/licenses/gpl.html ]
"
" This program is distributed in the hope that it will be useful,  but
" WITHOUT  ANY  WARRANTY;  without  even  the  implied   warranty   of
" MERCHANTABILITY or FITNESS FOR A PARTICULAR  PURPOSE.  See  the  GNU
" General Public License for more details.
"
" You should have received a copy of the GNU  General  Public  License
" along with  this  program;  if  not,  write  to  the  Free  Software
" Foundation,  Inc.,  59  Temple  Place  -  Suite  330,   Boston,   MA
" 02111-1307, USA.
"

" register as a Cream add-on
if exists("$CREAM")
	" don't list unless developer
	if !exists("g:cream_dev")
		finish
	endif

	call Cream_addon_register(
	\ 'Release', 
	\ 'Create Cream release package', 
	\ 'Create Cream release package', 
	\ 'Cream Devel.Release', 
	\ 'call Cream_release()',
	\ '<Nil>'
	\ )
endif

function! Cream_release()
" create a Cream release package from a checkout of CVS (g:cream_cvs)
" Note:  "mkdir" is multi-platform!

	" verify g:cream_cvs exists
	if !exists("g:cream_cvs")
		call confirm(
			\ "Error: g:cream_cvs not found. Quitting...\n" .
			\ "\n", "&Ok", 1, "Info")
		return
	endif

	if has("win32") && !has("win32unix")
		if executable("zip") == 0
			call confirm(
				\ "Unable to create final package, utility \"zip\" doesn't exist on the path.\n" .
				\ "\n", "&Ok", 1, "Info")
			return
		endif
	else
		" must have both
		if executable("tar") == 0
			call confirm(
				\ "Unable to create final package, utility \"tar\" doesn't exist on the path.\n" .
				\ "\n", "&Ok", 1, "Info")
			return
		endif
		if executable("gzip") == 0
			call confirm(
				\ "Unable to create final package, utility \"gzip\" doesn't exist on the path.\n" .
				\ "\n", "&Ok", 1, "Info")
			return
		endif
	endif

	" silent actions *related to copying* (optional)
	"   ""         -- shows prompts, shells
	"   "silent "  -- avoid prompts, shells
	"   "silent! " -- avoid prompts, shells and errors
	let mysilent = ""

	" platform-specifics vars
	if has("win32") && !has("win32unix")
		let slash = '\'
		let quote = '"'
		" very important!
		let myshellslash = &shellslash
		set noshellslash
		let copycmd = 'xcopy'
	else
		let slash = '/'
		let quote = ''
		let copycmd = 'cp -p'
	endif

    " use periods, Debian requires it
    "" don't use periods in version indication for file names (we want
    "" to match CVS tag)
    "let ver = substitute(g:cream_version_str, '\.', '-', 'g')
    let ver = g:cream_version_str 

	" make package directory 
	if has("win32") && !has("win32unix")
		" .\cream (two above module CVS hole)
		let releasedir = fnamemodify(g:cream_cvs, ":h:h") . slash . 'release-windows-' . strftime("%Y%m%dT%H%M%S")
	else
		" ./cream-X.XX (two above module CVS hole)
		let releasedir = fnamemodify(g:cream_cvs, ":h:h") . slash . 'release-unix-' . strftime("%Y%m%dT%H%M%S")
	endif
	execute ':' . mysilent . '!mkdir ' . quote . releasedir . quote

	" make package version directory (Unix only)
	if has("win32") && !has("win32unix")
        let releasedir = releasedir . slash . 'cream'
	else
		let releasedir = releasedir . slash . 'cream-' . ver
		execute ':' . mysilent . '!mkdir ' . quote . releasedir . quote
	endif

	" make package cream directory
	execute ':' . mysilent . '!mkdir ' . quote . releasedir . quote

	" make subdirectories
	execute ':' . mysilent . '!mkdir ' . quote . releasedir . slash . 'addons'     . quote
	execute ':' . mysilent . '!mkdir ' . quote . releasedir . slash . 'bitmaps'    . quote
	execute ':' . mysilent . '!mkdir ' . quote . releasedir . slash . 'docs'       . quote
	execute ':' . mysilent . '!mkdir ' . quote . releasedir . slash . 'docs-html'  . quote
	execute ':' . mysilent . '!mkdir ' . quote . releasedir . slash . 'help'       . quote
	execute ':' . mysilent . '!mkdir ' . quote . releasedir . slash . 'spelldicts' . quote
	execute ':' . mysilent . '!mkdir ' . quote . releasedir . slash . 'views'      . quote

	" copy files (don't just copy the structure, CVS interlaced)
	execute ':' . mysilent . '!' . copycmd . ' ' . quote . g:cream_cvs . slash . 'creamrc'     . quote . ' ' . quote . releasedir . quote
	execute ':' . mysilent . '!' . copycmd . ' ' . quote . g:cream_cvs . slash . '*.vim'       . quote . ' ' . quote . releasedir . quote
	execute ':' . mysilent . '!' . copycmd . ' ' . quote . g:cream_cvs . slash . 'cream.svg'   . quote . ' ' . quote . releasedir . quote
	if has("win32")
		execute ':' . mysilent . '!' . copycmd . ' ' . quote . g:cream_cvs . slash . 'cream.ico'     . quote . ' ' . quote . releasedir . quote
		execute ':' . mysilent . '!' . copycmd . ' ' . quote . g:cream_cvs . slash . 'INSTALL.bat'   . quote . ' ' . quote . releasedir . quote
	else
		execute ':' . mysilent . '!' . copycmd . ' ' . quote . g:cream_cvs . slash . 'cream.png'     . quote . ' ' . quote . releasedir . quote
		execute ':' . mysilent . '!' . copycmd . ' ' . quote . g:cream_cvs . slash . 'INSTALL.sh'    . quote . ' ' . quote . releasedir . quote
		execute ':' . mysilent . '!' . copycmd . ' ' . quote . g:cream_cvs . slash . 'cream'         . quote . ' ' . quote . releasedir . quote
		execute ':' . mysilent . '!' . copycmd . ' ' . quote . g:cream_cvs . slash . 'cream.desktop' . quote . ' ' . quote . releasedir . quote
	endif

	execute ':' . mysilent . '!' . copycmd . ' ' . quote . g:cream_cvs . slash . 'addons'     . slash . '*.vim'  . quote . ' ' . quote . releasedir . slash . 'addons' . quote
	if has("win32")
		let bitmap = '*.bmp'
	else
		let bitmap = '*.xpm'
	endif
	execute ':' . mysilent . '!' . copycmd . ' ' . quote . g:cream_cvs . slash . 'bitmaps'    . slash . bitmap                        . quote . ' ' . quote . releasedir . slash . 'bitmaps' . quote
	
	execute ':' . mysilent . '!' . copycmd . ' ' . quote . g:cream_cvs . slash . 'docs'       . slash . '*.txt'                       . quote . ' ' . quote . releasedir . slash . 'docs'       . quote
	execute ':' . mysilent . '!' . copycmd . ' ' . quote . g:cream_cvs . slash . 'docs-html'  . slash . '*.html'                      . quote . ' ' . quote . releasedir . slash . 'docs-html'  . quote
	execute ':' . mysilent . '!' . copycmd . ' ' . quote . g:cream_cvs . slash . 'docs-html'  . slash . '*.css'                       . quote . ' ' . quote . releasedir . slash . 'docs-html'  . quote
	execute ':' . mysilent . '!' . copycmd . ' ' . quote . g:cream_cvs . slash . 'docs-html'  . slash . '*.png'                       . quote . ' ' . quote . releasedir . slash . 'docs-html'  . quote
	execute ':' . mysilent . '!' . copycmd . ' ' . quote . g:cream_cvs . slash . 'help'       . slash . '*.txt'                       . quote . ' ' . quote . releasedir . slash . 'help'       . quote
	execute ':' . mysilent . '!' . copycmd . ' ' . quote . g:cream_cvs . slash . 'spelldicts' . slash . 'cream-spell-dict-eng-s*.vim' . quote . ' ' . quote . releasedir . slash . 'spelldicts' . quote
	" don't copy cream/views

	" create package
	if has("win32") && !has("win32unix")
		execute 'cd ' . fnamemodify(releasedir, ":h")
		execute ':!zip -r ' . quote . fnamemodify(releasedir, ":h") . slash . 'cream-' . ver . '.zip' . quote . ' ' . quote . 'cream' . quote
	else
		execute ':!cd ' . fnamemodify(releasedir, ":h") . '; tar -cvf cream-' . ver . '.tar ' . 'cream-' . ver
		execute ':!gzip ' . fnamemodify(releasedir, ":h") . slash . 'cream-' . ver . '.tar'
		" Warning
		call confirm(
			\ "VERIFY!\n" .
			\ " o Permissions on package files and directories\n" .
			\ " o Line endings on files\n" .
			\ "\n", "&Ok", 1, "Info")
	endif

	if exists("myshellslash")
		let &shellslash = myshellslash
	endif

endfunction

